export declare type UpdateDetail = {
    parent?: object;
    name?: string;
    oldValue: any;
    newValue: any;
};
export declare type JsonEditProps<T> = {
    data: T;
    name?: string;
    parent?: object;
    updateValue?: (value: T) => void;
    onChange?: (ev: Event, detail: UpdateDetail) => void;
    editElement?: boolean;
};
export declare type PrecisionType = 'minutes' | 'seconds' | 'milliseconds';
/**
 * functions inside can be overridden
 * */
export declare namespace JsonEditRenders {
    let precision: PrecisionType;
    function string(props: JsonEditProps<string>): JSX.Element;
    function number(props: JsonEditProps<number>): JSX.Element;
    function bigint(props: JsonEditProps<bigint>): JSX.Element;
    function boolean(props: JsonEditProps<boolean>): JSX.Element;
    function Function(props: JsonEditProps<Function>): JSX.Element;
    function symbol(props: JsonEditProps<symbol>): JSX.Element[];
    function undefined(props: JsonEditProps<undefined>): JSX.Element;
    function Null(props: JsonEditProps<undefined>): JSX.Element;
    function dateString(date: Date): string;
    function timeString(date: Date): string;
    function date(props: JsonEditProps<number | Date>): JSX.Element[];
    function set(props: JsonEditProps<Set<any>>): JSX.Element;
    function map(props: JsonEditProps<Map<any, any>>): JSX.Element;
    function array(props: JsonEditProps<any[]>): JSX.Element;
    function text(props: JsonEditProps<Text>): JSX.Element;
    function attributes(props: JsonEditProps<HTMLElement>): HTMLElement;
    function htmlElement(props: JsonEditProps<HTMLElement>): JSX.Element;
    function object(props: JsonEditProps<object>): JSX.Element;
    function unknown(props: JsonEditProps<unknown>): JSX.Element;
}
/**
 * supported data type:
 *   + HTMLElement
 *   + Text (DOM node)
 *   + string
 *   + number
 *   + timestamp (in number)
 *   + Date
 *   + bigint
 *   + boolean
 *   + function (show the source code)
 *   + symbol
 *   + undefined
 *   + null
 *   + Set
 *   + Map
 *   + Array
 *   + object
 * */
export declare function JsonEdit(props: JsonEditProps<any>): HTMLElement | HTMLElement[];
