"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const s_js_1 = tslib_1.__importDefault(require("s-js"));
const Surplus = tslib_1.__importStar(require("surplus"));
const helpers_1 = require("./helpers");
const json_view_1 = require("./json-view");
// tslint:disable-next-line no-unused-expression
Surplus;
/**
 * functions inside can be overridden
 * */
var JsonEditRenders;
(function (JsonEditRenders) {
    JsonEditRenders.precision = 'minutes';
    function updateValue(props, newValue) {
        if (props.updateValue) {
            props.updateValue(newValue);
            return;
        }
        if (!props.parent) {
            console.warn('missing parent');
            props.data = newValue;
            return;
        }
        if (props.name === void 0) {
            console.warn('missing name');
            props.data = newValue;
            return;
        }
        props.parent[props.name] = newValue;
    }
    function update(ev, props, newValue) {
        const oldValue = props.data;
        updateValue(props, newValue);
        if (props.onChange) {
            props.onChange(ev, {
                parent: props.parent,
                name: props.name,
                oldValue,
                newValue,
            });
        }
    }
    function string(props) {
        return <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='text' value={props.data} onChange={ev => update(ev, props, ev.target.value)}/>;
    }
    JsonEditRenders.string = string;
    function number(props) {
        return <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='number' value={props.data} onChange={ev => update(ev, props, ev.target.valueAsNumber)}/>;
    }
    JsonEditRenders.number = number;
    function bigint(props) {
        return <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='text' value={props.data.toString()} onChange={ev => update(ev, props, BigInt(ev.target.value))}/>;
    }
    JsonEditRenders.bigint = bigint;
    function boolean(props) {
        return <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='checkbox' checked={props.data} onChange={ev => update(ev, props, ev.target.checked)}/>;
    }
    JsonEditRenders.boolean = boolean;
    // tslint:disable-next-line ban-types
    function Function(props) {
        return <textarea name={helpers_1.JsonHelpers.toStringName(props.name)} value={props.data.toString()} 
        // tslint:disable-next-line no-eval
        onChange={ev => update(ev, props, eval(ev.target.value))}/>;
    }
    JsonEditRenders.Function = Function;
    function symbol(props) {
        return [
            <label>Symbol:</label>,
            <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='text' value={Symbol.keyFor(props.data)} onChange={ev => update(ev, props, Symbol.for(ev.target.value))}/>,
        ];
    }
    JsonEditRenders.symbol = symbol;
    function undefined(props) {
        // TODO infer type for undefined?
        return json_view_1.JsonViewRenders.undefined(props);
    }
    JsonEditRenders.undefined = undefined;
    function Null(props) {
        // TODO infer type for null?
        return json_view_1.JsonViewRenders.Null(props);
    }
    JsonEditRenders.Null = Null;
    function d2(x) {
        if (x < 10) {
            return '0' + x;
        }
        return x.toString();
    }
    function d3(x) {
        if (x < 10) {
            return '0' + x;
        }
        if (x < 100) {
            return '00' + x;
        }
        return x.toString();
    }
    // input[type=date].value string format in local timezone
    function dateString(date) {
        return `${date.getFullYear()}-${d2(date.getMonth() + 1)}-${d2(date.getDate())}`;
    }
    JsonEditRenders.dateString = dateString;
    // input[type=time].value string format in local timezone
    function timeString(date) {
        let timeStr = `${d2(date.getHours())}:${d2(date.getMinutes())}`;
        switch (JsonEditRenders.precision) {
            case 'seconds':
            case 'milliseconds':
                timeStr += ':' + d2(date.getSeconds());
                if (JsonEditRenders.precision === 'milliseconds') {
                    timeStr += '.' + d3(date.getMilliseconds());
                }
        }
        return timeStr;
    }
    JsonEditRenders.timeString = timeString;
    function date(props) {
        // TODO
        let date = new Date(props.data);
        const dateStr = dateString(date);
        const timeStr = timeString(date);
        return [
            <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='date' value={dateStr} onChange={ev => {
                const value = ev.target.value;
                if (value) {
                    date = new Date(value + ' ' + timeStr);
                }
                else {
                    // not date but still has time
                    const input = document.createElement('input');
                    input.type = 'time';
                    input.value = timeStr;
                    date = input.valueAsDate;
                }
                if (props.data instanceof Date) {
                    update(ev, props, date);
                }
                else {
                    update(ev, props, date.getTime());
                }
            }}/>,
            <input name={helpers_1.JsonHelpers.toStringName(props.name)} type='time' value={timeStr} onChange={ev => {
                const value = ev.target.value;
                if (value) {
                    date = new Date(dateStr + ' ' + value);
                }
                else {
                    // update time but still has date
                    date = new Date(dateStr);
                }
                if (props.data instanceof Date) {
                    update(ev, props, date);
                }
                else {
                    update(ev, props, date.getTime());
                }
            }}/>,
        ];
    }
    JsonEditRenders.date = date;
    function set(props) {
        const signals = Array.from(props.data).map(x => s_js_1.default.data(x));
        return <ul>{signals.map((signal) => <li>{JsonEdit(Object.assign(Object.assign({}, props), { parent: props.data, data: s_js_1.default.sample(signal), updateValue: newValue => {
                props.data.delete(signal());
                props.data.add(newValue);
                signal(newValue);
            } }))}</li>)}</ul>;
    }
    JsonEditRenders.set = set;
    function map(props) {
        const signals = Array.from(props.data.entries()).map(x => s_js_1.default.data(x));
        return <table>
      <tbody>
      {signals.map((kvSignal) => {
            return <tr>
          <td>{JsonEdit(Object.assign(Object.assign({}, props), { parent: props.data, data: s_js_1.default.sample(kvSignal)[0], updateValue: newKey => {
                    const [key, value] = kvSignal();
                    props.data.delete(key);
                    props.data.set(newKey, value);
                    kvSignal([newKey, value]);
                } }))}</td>
          <td>=></td>
          <td>{JsonEdit(Object.assign(Object.assign({}, props), { parent: props.data, name: kvSignal()[0], data: kvSignal()[1], updateValue: newValue => {
                    const [key] = kvSignal();
                    props.data.set(key, newValue);
                    kvSignal([key, newValue]);
                } }))}</td>
        </tr>;
        })}
      </tbody>
    </table>;
    }
    JsonEditRenders.map = map;
    function array(props) {
        return <ol>{props.data.map((x, i) => <li>{JsonEdit(Object.assign(Object.assign({}, props), { parent: props.data, data: x }))}</li>)}</ol>;
    }
    JsonEditRenders.array = array;
    function asString(props, options) {
        return string(Object.assign(Object.assign({}, props), options));
    }
    function text(props) {
        return json_view_1.JsonViewRenders.table({
            textContent: asString(props, {
                name: 'textContent',
                data: props.data.textContent || '',
                updateValue: value => props.data.textContent = value,
            }),
        });
    }
    JsonEditRenders.text = text;
    function attributes(props) {
        const table = document.createElement('table');
        const tbody = document.createElement('tbody');
        const attrs = props.data.attributes;
        for (let i = 0; i < attrs.length; i++) {
            const attr = attrs.item(i);
            if (attr === null) {
                continue;
            }
            tbody.appendChild(<tr>
        <td>{string(Object.assign(Object.assign({}, props), { parent: attr, name: 'attributes', data: attr.name, updateValue: newName => {
                    const value = attr.value;
                    attrs.removeNamedItem(attr.name);
                    props.data.setAttribute(newName, value);
                } }))}</td>
        <td>:</td>
        <td>{string(Object.assign(Object.assign({}, props), { parent: attr, name: attr.name, data: attr.value, updateValue: newValue => {
                    props.data.setAttribute(attr.name, newValue);
                } }))}</td>
      </tr>);
        }
        table.appendChild(tbody);
        return table;
    }
    JsonEditRenders.attributes = attributes;
    function htmlElement(props) {
        const e = props.data;
        const children = [];
        e.childNodes.forEach(child => children.push(JsonEdit(Object.assign(Object.assign({}, props), { data: child }))));
        return json_view_1.JsonViewRenders.table({
            tagName: json_view_1.JsonViewRenders.string({ name: 'tagName', data: e.tagName }),
            id: asString(props, { name: 'id', data: e.id, updateValue: value => props.data.id = value }),
            className: asString(props, { name: 'className', data: e.className, updateValue: value => props.data.className = value }),
            attributes: attributes(props),
            nodeValue: asString(props, { name: 'nodeValue', data: e.nodeValue || '', updateValue: value => props.data.nodeValue = value }),
            value: JsonEdit(Object.assign(Object.assign({}, props), { name: 'value', data: e.value, updateValue: value => props.data.value = value })),
            childNodes: array(Object.assign(Object.assign({}, props), { editElement: false, name: 'childNodes', data: children, updateValue: value => {
                    console.log('update childNodes', value);
                } })),
        });
    }
    JsonEditRenders.htmlElement = htmlElement;
    function object(props) {
        const signals = Object.entries(props.data).map(x => s_js_1.default.data(x));
        return <table>
      <tbody>{signals.map((kvSignal) => {
            return <tr>
          <td>{string(Object.assign(Object.assign({}, props), { parent: props.data, data: s_js_1.default.sample(kvSignal)[0], updateValue: newKey => {
                    const [key, value] = kvSignal();
                    delete props.data[key];
                    props.data[newKey] = value;
                    kvSignal([newKey, value]);
                } }))}</td>
          <td>:</td>
          <td>{JsonEdit(Object.assign(Object.assign({}, props), { parent: props.data, name: kvSignal()[0], data: kvSignal()[1] }))}</td>
        </tr>;
        })}</tbody>
    </table>;
    }
    JsonEditRenders.object = object;
    function unknown(props) {
        // TODO infer type for unknown?
        return json_view_1.JsonViewRenders.unknown(props);
    }
    JsonEditRenders.unknown = unknown;
})(JsonEditRenders = exports.JsonEditRenders || (exports.JsonEditRenders = {}));
const r = JsonEditRenders;
/**
 * supported data type:
 *   + HTMLElement
 *   + Text (DOM node)
 *   + string
 *   + number
 *   + timestamp (in number)
 *   + Date
 *   + bigint
 *   + boolean
 *   + function (show the source code)
 *   + symbol
 *   + undefined
 *   + null
 *   + Set
 *   + Map
 *   + Array
 *   + object
 * */
function JsonEdit(props) {
    const data = props.data;
    const type = typeof data;
    switch (type) {
        case 'string':
            return r.string(props);
        case 'number':
            if (helpers_1.JsonHelpers.isTimeName(props.name)) {
                return r.date(props);
            }
            else {
                return r.number(props);
            }
        case 'bigint':
            return r.bigint(props);
        case 'boolean':
            return r.boolean(props);
        case 'function':
            return r.Function(props);
        case 'symbol':
            return r.symbol(props);
        case 'undefined':
            return r.undefined(props);
        case 'object':
            if (data === null) {
                return r.Null(props);
            }
            if (data instanceof Date) {
                return r.date(props);
            }
            if (data instanceof Set) {
                return r.set(props);
            }
            if (data instanceof Map) {
                return r.map(props);
            }
            if (Array.isArray(data)) {
                return r.array(props);
            }
            if (props.editElement) {
                if (data instanceof Text) {
                    return r.text(props);
                }
                if (data instanceof HTMLElement) {
                    return r.htmlElement(props);
                }
            }
            if (data instanceof Text) {
                return props.data;
            }
            if (data instanceof HTMLElement) {
                return props.data;
            }
            return r.object(props);
        default: {
            const x = type;
            console.error('unknown data type:', x);
            return r.unknown(props);
        }
    }
}
exports.JsonEdit = JsonEdit;
//# sourceMappingURL=json-edit.jsx.map