export declare type JsonViewProps<T> = {
    data: T;
    name?: string;
    expandElement?: boolean;
};
/**
 * functions inside can be overridden
 * */
export declare namespace JsonViewRenders {
    function date(props: JsonViewProps<Date>): JSX.Element;
    function number(props: JsonViewProps<number>): JSX.Element;
    function set(prop: JsonViewProps<Set<any>>): JSX.Element;
    function map(props: JsonViewProps<Map<any, any>>): JSX.Element;
    function array(props: JsonViewProps<any[]>): JSX.Element;
    function table(data: Record<string, HTMLElement | Text>): JSX.Element;
    function object(props: JsonViewProps<object>): JSX.Element;
    function string(props: JsonViewProps<string>): JSX.Element;
    function text(props: JsonViewProps<Text>): JSX.Element;
    function attributes(attrs: NamedNodeMap): object;
    function htmlElement(props: JsonViewProps<HTMLElement>): JSX.Element;
    function bigint(props: JsonViewProps<bigint>): JSX.Element;
    function boolean(props: JsonViewProps<boolean>): JSX.Element;
    function Function(props: JsonViewProps<Function>): JSX.Element;
    function symbol(props: JsonViewProps<symbol>): JSX.Element;
    function Null(props: JsonViewProps<any>): JSX.Element;
    function undefined(props: JsonViewProps<undefined>): JSX.Element;
    function unknown(props: JsonViewProps<unknown>): JSX.Element;
}
/**
 * supported data type:
 *   + HTMLElement
 *   + Text (DOM node)
 *   + string
 *   + number
 *   + timestamp (in number)
 *   + Date
 *   + bigint
 *   + boolean
 *   + function (show the source code)
 *   + symbol
 *   + undefined
 *   + null
 *   + Set
 *   + Map
 *   + Array
 *   + object
 * */
export declare function JsonView(props: JsonViewProps<any>): HTMLElement;
