"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Surplus = tslib_1.__importStar(require("surplus"));
const helpers_1 = require("./helpers");
// tslint:disable-next-line no-unused-expression
Surplus;
/**
 * functions inside can be overridden
 * */
var JsonViewRenders;
(function (JsonViewRenders) {
    function date(props) {
        return string(Object.assign(Object.assign({}, props), { data: props.data.toLocaleString() }));
    }
    JsonViewRenders.date = date;
    function number(props) {
        return string(Object.assign(Object.assign({}, props), { data: props.data.toLocaleString() }));
    }
    JsonViewRenders.number = number;
    function set(prop) {
        return <ul>
      {Array.from(prop.data).map(x => <li>
        <JsonView {...prop} data={x}/>
      </li>)}
    </ul>;
    }
    JsonViewRenders.set = set;
    function map(props) {
        return <table>
      <tbody>
      {Array.from(props.data.entries()).map(([key, value]) => <tr>
        <td><JsonView {...props} data={key}/></td>
        <td>=></td>
        <td><JsonView {...props} data={value} name={key}/></td>
      </tr>)}
      </tbody>
    </table>;
    }
    JsonViewRenders.map = map;
    function array(props) {
        return <ol>
      {props.data.map(x => <li>
        <JsonView {...props} data={x}/>
      </li>)}
    </ol>;
    }
    JsonViewRenders.array = array;
    function table(data) {
        return <table>
      <tbody>
      {Object.entries(data).map(([key, value]) => <tr>
        <td>{string({ data: key })}:</td>
        <td>{value}</td>
      </tr>)}
      </tbody>
    </table>;
    }
    JsonViewRenders.table = table;
    function object(props) {
        return <table>
      <tbody>
      {Object.entries(props.data).map(([key, value]) => <tr>
        <td>{string(Object.assign(Object.assign({}, props), { data: key }))}:</td>
        <td><JsonView {...props} data={value} name={key}/></td>
      </tr>)}
      </tbody>
    </table>;
    }
    JsonViewRenders.object = object;
    function string(props) {
        return <span>{props.data}</span>;
    }
    JsonViewRenders.string = string;
    function text(props) {
        const textContent = props.data.textContent;
        return table({
            textContent: textContent === null ? Null(props) :
                string(Object.assign(Object.assign({}, props), { data: textContent })),
        });
    }
    JsonViewRenders.text = text;
    function attributes(attrs) {
        const res = {};
        for (let i = 0; i < attrs.length; i++) {
            const attr = attrs.item(i);
            if (attr === null) {
                continue;
            }
            res[attr.name] = attr.value;
        }
        return res;
    }
    JsonViewRenders.attributes = attributes;
    function htmlElement(props) {
        const e = props.data;
        const res = {
            tagName: e.tagName.toLocaleLowerCase(),
            id: e.id,
            className: e.className,
            attributes: e.attributes.length > 0 ? attributes(e.attributes) : void 0,
            nodeValue: e.nodeValue,
            value: e.value,
        };
        Object.keys(res).forEach((_key) => {
            const key = _key;
            const value = res[key];
            switch (value) {
                case '':
                case null:
                case void 0:
                    delete res[key];
                    break;
                default:
                    if (Array.isArray(value) && value.length === 0) {
                        delete res[key];
                    }
            }
        });
        return object({ data: res });
    }
    JsonViewRenders.htmlElement = htmlElement;
    function bigint(props) {
        return string(Object.assign(Object.assign({}, props), { data: props.data.toLocaleString() }));
    }
    JsonViewRenders.bigint = bigint;
    function boolean(props) {
        return string(Object.assign(Object.assign({}, props), { data: props.data ? 'yes' : 'no' }));
    }
    JsonViewRenders.boolean = boolean;
    // tslint:disable-next-line ban-types
    function Function(props) {
        return string(Object.assign(Object.assign({}, props), { data: props.data.toString() }));
    }
    JsonViewRenders.Function = Function;
    function symbol(props) {
        return string(Object.assign(Object.assign({}, props), { data: props.data.toString() }));
    }
    JsonViewRenders.symbol = symbol;
    function Null(props) {
        return string(Object.assign(Object.assign({}, props), { data: 'null' }));
    }
    JsonViewRenders.Null = Null;
    function undefined(props) {
        return string(Object.assign(Object.assign({}, props), { data: 'undefined' }));
    }
    JsonViewRenders.undefined = undefined;
    function unknown(props) {
        return string(Object.assign(Object.assign({}, props), { data: Object.prototype.toString.call(props.data) }));
    }
    JsonViewRenders.unknown = unknown;
})(JsonViewRenders = exports.JsonViewRenders || (exports.JsonViewRenders = {}));
const r = JsonViewRenders;
/**
 * supported data type:
 *   + HTMLElement
 *   + Text (DOM node)
 *   + string
 *   + number
 *   + timestamp (in number)
 *   + Date
 *   + bigint
 *   + boolean
 *   + function (show the source code)
 *   + symbol
 *   + undefined
 *   + null
 *   + Set
 *   + Map
 *   + Array
 *   + object
 * */
function JsonView(props) {
    const data = props.data;
    const type = typeof data;
    switch (type) {
        case 'string':
            return r.string(props);
        case 'number':
            if (helpers_1.JsonHelpers.isTimeName(props.name)) {
                return r.date(Object.assign(Object.assign({}, props), { data: new Date(data) }));
            }
            else {
                return r.number(props);
            }
        case 'bigint':
            return r.bigint(props);
        case 'boolean':
            return r.boolean(props);
        case 'function':
            return r.Function(props);
        case 'symbol':
            return r.symbol(props);
        case 'undefined':
            return r.undefined(props);
        case 'object':
            if (data === null) {
                return r.Null(props);
            }
            if (data instanceof Date) {
                return r.date(props);
            }
            if (data instanceof Set) {
                return r.set(props);
            }
            if (data instanceof Map) {
                return r.map(props);
            }
            if (Array.isArray(data)) {
                return r.array(props);
            }
            if (props.expandElement) {
                if (data instanceof Text) {
                    return r.text(props);
                }
                if (data instanceof HTMLElement) {
                    return r.htmlElement(props);
                }
            }
            if (data instanceof Text) {
                return data;
            }
            if (data instanceof HTMLElement) {
                return data;
            }
            return r.object(props);
        default: {
            const x = type;
            console.error('unknown data type:', x);
            return r.unknown(props);
        }
    }
}
exports.JsonView = JsonView;
//# sourceMappingURL=json-view.jsx.map